# -*- coding: utf-8 -*-
"""
Script to cache anonymous houseprint data into hp_anonymous.pkl

Created on 05/07/2014 by Roel De Coninck
"""

from opengrid.library.houseprint import houseprint

##############################################################################

hp = houseprint.Houseprint()
print('Sensor data fetched')

hp.save('hp_anonymous.pkl')
print('Houseprint saved to hp_anonymous.pkl')



# -*- coding: utf-8 -*-
"""
Synchronize the opengrid data to your computer.

Created on 16/12/2014 by Roel De Coninck
"""

from opengrid.library import houseprint

hp = houseprint.Houseprint()
hp.sync_tmpos()




# coding: utf-8

# ## This script shows the visualization of electricity, water and gas consumption using carpet plots
# 
# To get started, first run the 'Synchronize data' script
# 
# #### Imports and paths

# In[ ]:

import os
import sys
import pytz
import time
import inspect
import numpy as np
import pandas as pd
import datetime as dt
import tmpo

from opengrid import config
from opengrid.library import plotting
from opengrid.library import houseprint

c=config.Config()

try:
    if os.path.exists(c.get('tmpo', 'data')):
        path_to_tmpo_data = c.get('tmpo', 'data')
except:
    path_to_tmpo_data = None

# configuration for the plots
DEV = c.get('env', 'type') == 'dev' # DEV is True if we are in development environment, False if on the droplet
print("Environment configured for development: {}".format(DEV))
if not DEV:
    # production environment: don't try to display plots
    import matplotlib
    matplotlib.use('Agg')

import matplotlib.pyplot as plt
from matplotlib.dates import MinuteLocator, HourLocator, DateFormatter, AutoDateLocator, num2date

if DEV:
    if c.get('env', 'plots') == 'inline':
        get_ipython().magic(u'matplotlib inline')
    else:
        get_ipython().magic(u'matplotlib qt')
else:
    pass # don't try to render plots

plt.rcParams['figure.figsize'] = 16,8

# path to data
path_to_data = c.get('data', 'folder')
if not os.path.exists(path_to_data):
    raise IOError("Provide your path to the data in your config.ini file. ")
else:
    path_to_fig = os.path.join(path_to_data, 'figures')
    if not os.path.isdir(path_to_fig): os.makedirs(path_to_fig)


# In[ ]:

c.get('data','folder')


# ### Loading meta data and user variables

# In[ ]:

hp = houseprint.Houseprint()

end = pd.Timestamp(time.time(), unit='s')
start = end - pd.Timedelta('21 days')


# In[ ]:

hp.save('new_houseprint.pkl')


# ## Plotting

# In[ ]:

#hp.sync_tmpos()


# ### Water sensors

# In[ ]:

water_sensors = hp.get_sensors(sensortype='water')
print("{} water sensors".format(len(water_sensors)))


# In[ ]:

for sensor in water_sensors:
    ts = sensor.get_data(head=start, tail=end)
    if not ts.dropna().empty:
        plotting.carpet(ts, title=' - '.join([sensor.device.key, sensor.description, sensor.key]), zlabel=r'Flow [l/min]')
        plt.savefig(os.path.join(path_to_fig, 'carpet_'+sensor.type+'_'+sensor.key), dpi=100)
        if not DEV:
            plt.close()


# ### Gas sensors

# In[ ]:

gas_sensors = hp.get_sensors(sensortype=('gas'))
print("{} gas sensors".format(len(gas_sensors)))


# In[ ]:

for sensor in gas_sensors:
    ts = sensor.get_data(head=start, tail=end)
    if not ts.dropna().empty:
        plotting.carpet(ts, title=' - '.join([sensor.device.key, sensor.description, sensor.key]), zlabel=r'Gas consumption [W]')
        plt.savefig(os.path.join(path_to_fig, 'carpet_'+sensor.type+'_'+sensor.key), dpi=100)
        if not DEV:
            plt.close()


# ### Electricity sensors

# In[ ]:

elec_sensors = hp.get_sensors(sensortype=('electricity'))
print("{} electricity sensors".format(len(elec_sensors)))


# In[ ]:

for sensor in elec_sensors:
    ts = sensor.get_data(head=start, tail=end)
    if not ts.dropna().empty:
        plotting.carpet(ts, title=' - '.join([sensor.device.key, sensor.description, sensor.key]), zlabel=r'Power [W]')
        plt.savefig(os.path.join(path_to_fig, 'carpet_'+sensor.type+'_'+sensor.key), dpi=100)
        if not DEV:
            plt.close()


# In[ ]:






__author__ = 'Jan Pecinovsky'

import datetime as dt
import forecastio
from forecastio.models import Forecast
from geopy import Location, Point, GoogleV3
import numpy as np
import pandas as pd
import pytz
from cached_property import cached_property
from tqdm import tqdm
import os
import pickle


from .misc import dayset, calculate_temperature_equivalent, \
    calculate_degree_days
from opengrid import config
cfg = config.Config()


class Weather():
    """
        Object that contains Weather Data from Forecast.io for multiple days as a Pandas Dataframe.
        NOTE: Forecast.io allows 1000 requests per day, after that you have to pay. Each requested day is 1 request.
    """

    def __init__(self, location, start, end=None, tz=None, cache=True):
        """
            Constructor

            Parameters
            ----------
            location : str | tuple(float, float)
                String can be City, address, POI. tuple = (lat, lng)
            start : datetime.datetime | pandas.Timestamp
                start of the interval to be searched
            end : datetime.datetime | pandas.Timestamp, optional
                end of the interval to be searched
                if None, use current time
            tz : str, optional
                tz specifies the timezone of the response.
                If none, response will be in the timezone of the location
            cache : bool
                use the cache or not
        """
        self.api_key = cfg.get('Forecast.io', 'apikey')
        self._location = location
        self._start = start
        self._end = end
        self._tz = tz
        self.cache = cache

        self._forecasts = []

    @cached_property  # so we don't have to lookup every time it is called
    def location(self):
        """
        Get the location, but as a geopy location object

        Returns
        -------
        Location
        """
        # if the input was a string, we do a google lookup
        if isinstance(self._location, str):
            location = GoogleV3().geocode(self._location)

        # if the input was an iterable, it is latitude and longitude
        elif hasattr(self._location, '__iter__'):
            lat, long = self._location
            gepoint = Point(latitude=lat, longitude=long)
            location = Location(point=gepoint)

        else:
            raise ValueError('Invalid location')

        return location

    @property
    def start(self):
        """
        Return localized start time

        Returns
        -------
        datetime.datetime | pandas.Timestamp
        """
        if self._start.tzinfo is None:
            return self.tz.localize(self._start)
        else:
            return self._start

    @property
    def end(self):
        """
        Return localized end time

        Returns
        -------
        datetime.datetime | pandas.Timestamp
        """
        if self._end is None:
            return pd.Timestamp('now', tz=self.tz.zone)
        elif self._end.tzinfo is None:
            return self.tz.localize(self._end)
        else:
            return self._end

    @property
    def tz(self):
        """
        Get the timezone

        Returns
        -------
        pytz.timezone
        """

        # if the user has specified a zone, use that one
        if self._tz is not None:
            tz = self._tz

        # if there already are some forecasts, the timezone is in there
        elif self._forecasts:
            tz = self._lookup_timezone()

        # use Google geocoder to lookup timezone
        else:
            lat, long, _alt = self.location.point
            tz = GoogleV3().timezone(location=(lat, long)).zone

        # return as a pytz object
        return pytz.timezone(tz)

    @property
    def forecasts(self):
        """
        Get a list of all forecast objects

        Returns
        -------
        list(Forecast)
        """
        # we stick the list in self._forecasts, only if it is still empty
        if not self._forecasts:
            # get list of seperate days
            days = dayset(start=self.start, end=self.end)
            # wrap in a tqdm so we get the progress bar
            for date in tqdm(days):
                self._forecasts.append(self._get_forecast(date=date))

        return self._forecasts

    def days(self,
             include_average_temperature=True,
             include_temperature_equivalent=True,
             include_heating_degree_days=True,
             heating_base_temperatures=[16.5],
             include_cooling_degree_days=True,
             cooling_base_temperatures=[18],
             include_daytime_cloud_cover=True,
             include_daytime_temperature=True
             ):
        """
        Create a dataframe with all weather data in daily resolution

        Parameters
        ----------
        include_average_temperature : bool (optional, default: True)
            Include automatic calculation of average temperature from hourly data
        include_temperature_equivalent : bool (optional, default: True)
            Include Temperature Equivalent to dataframe
        include_heating_degree_days : bool (optional, default: True)
            Add heating degree days to the dataframe
        heating_base_temperatures : list of numbers (optional, default 16.5)
            List of possible base temperatures for which to calculate heating degree days
        include_cooling_degree_days : bool (optional, default: False)
            Add cooling degree days to the dataframe
        cooling_base_temperatures : list of numbers (optional, default 18)
            List of possible base temperatures for which to calculate cooling degree days
        include_daytime_cloud_cover : bool (optional, default: True)
            Include average Cloud Cover during daytime hours (from sunrise to sunset)
        include_daytime_temperature : bool (optional, default: True)
            Include average Temperature during daytime hours (from sunrise to sunset)

        Returns
        -------
        Pandas Dataframe
        """
        if include_heating_degree_days or include_cooling_degree_days:
            include_temperature_equivalent = True

        if include_temperature_equivalent:
            include_average_temperature = True

        # if temperature_equivalent is needed,
        # we need to add 2 days before the start
        if include_temperature_equivalent:
            self._add_forecast(self.start - pd.Timedelta(days=1))
            self._add_forecast(self.start - pd.Timedelta(days=2))

        day_list = [self._forecast_to_day_series(forecast=forecast,
                                                 include_average_temperature=include_average_temperature,
                                                 include_daytime_cloud_cover=include_daytime_cloud_cover,
                                                 include_daytime_temperature=include_daytime_temperature
                                                 ) for forecast in self.forecasts]

        frame = pd.concat(day_list)
        frame = self._fix_index(frame).sort_index()

        if include_temperature_equivalent:
            frame['temperatureEquivalent'] = calculate_temperature_equivalent(temperatures=frame.temperature)
            frame.dropna(subset=['temperatureEquivalent'], inplace=True)

        if include_heating_degree_days:
            for base in heating_base_temperatures:
                frame['heatingDegreeDays{}'.format(base)] = calculate_degree_days(
                    temperature_equivalent=frame.temperatureEquivalent, base_temperature=base
                )

        if include_cooling_degree_days:
            for base in cooling_base_temperatures:
                frame['coolingDegreeDays{}'.format(base)] = calculate_degree_days(
                    temperature_equivalent=frame.temperatureEquivalent, base_temperature=base, cooling=True
                )

        return frame

    def hours(self):
        """
        Create a dataframe with all weather data in hourly resolution

        Returns
        -------
        Pandas Dataframe
        """
        day_list = [self._forecast_to_hour_series(forecast) for forecast in self.forecasts]
        frame = pd.concat(day_list)
        return self._fix_index(frame).sort_index().truncate(before=self.start, after=self.end)

    def _get_forecast(self, date):
        """
            Get the raw forecast object for a given date

            Parameters
            ----------
            date : datetime.date

            Returns
            -------
            forecastio forecast
        """
        # Forecast takes a dt.datetime
        # conversion from dt.date to dt.datetime, there must be a better way, right?
        time = dt.datetime(year=date.year, month=date.month, day=date.day)

        f = None
        if self.cache:
            f = self._load_from_cache(date)

        if not f:
            f = forecastio.load_forecast(key=self.api_key,
                                         lat=self.location.latitude,
                                         lng=self.location.longitude,
                                         time=time)
            if self.cache:
                self._save_in_cache(f, date)

        return f

    def _get_forecast_dates(self):
        """
        Return a set with the dates of all forecasts in self.forecasts

        Returns
        -------
        set of datetime.date
        """
        dates = []
        for forecast in self.forecasts:
            time = forecast.currently().time

            # the time is in UTC, we need to localize it.
            tz = pytz.timezone(self._lookup_timezone())
            time_utc = tz.fromutc(time)

            dates.append(time_utc.date())
        return set(dates)

    def _add_forecast(self, date):
        """
        Add a forecast to the list of forecasts

        Parameters
        ----------
        date : dt.date | dt.datetime | pd.Timestamp
        """
        # for if you pass a datetime instead of a date
        if hasattr(date, 'date'):
            date = date.date()

        if date not in self._get_forecast_dates():
            self._forecasts.append(self._get_forecast(date))

    def _forecast_to_hour_series(self, forecast):
        """
        Transforms the hourly data of a forecast object to a pandas dataframe

        Parameters
        ----------
        forecast : Forecast object

        Returns
        -------
        Pandas Dataframe

        """
        hour_list = [pd.Series(hour.d) for hour in forecast.hourly().data]
        frame = pd.concat(hour_list, axis=1).T
        return frame

    def _fix_index(self, frame):
        """
        Transform a column of a dataframe to a datetimeindex, set as the index and localize

        Parameters
        ----------
        frame : Pandas Dataframe

        Returns
        -------
        Pandas Dataframe

        """
        frame['time'] = pd.DatetimeIndex(frame['time'].astype('datetime64[s]'))
        frame.set_index('time', inplace=True)
        frame = frame.tz_localize('UTC')
        frame = frame.tz_convert(self.tz.zone)
        return frame

    def _lookup_timezone(self):
        """
        Lookup the timezone in the JSON of the first forecast

        Returns
        -------
        String (Pytz timezone)
        """
        tz = self.forecasts[0].json['timezone']
        return tz

    def _forecast_to_day_series(
            self,
            forecast,
            include_average_temperature,
            include_daytime_cloud_cover,
            include_daytime_temperature
    ):
        """
        Transforms the daily data of a forecast object to a pandas dataframe

        Parameters
        ----------
        include_daytime_cloud_cover : bool
        include_daytime_temperature : bool
        include_average_temperature : bool
        forecast : Forecast Object

        Returns
        -------
        Pandas dataframe

        """
        data = forecast.daily().data[0].d

        if include_average_temperature:
            average_temperature = self._get_daily_average(forecast=forecast, key='temperature')
            data.update({'temperature': average_temperature})
        if include_daytime_cloud_cover:
            daytime_cloud_cover = self._get_daytime_average(forecast=forecast, key='cloudCover')
            data.update({'daytimeCloudCover': daytime_cloud_cover})
        if include_daytime_temperature:
            daytime_temperature = self._get_daytime_average(forecast=forecast, key='temperature')
            data.update({'daytimeTemperature': daytime_temperature})

        frame = [pd.Series(data)]
        return pd.concat(frame, axis=1).T

    def _get_daily_average(self, forecast, key):
        """
        Calculate the average daily value for a given forecast and a given key from the hourly values

        Parameters
        ----------
        forecast : Forecast object
        key : String

        Returns
        -------
        float
        """
        # make a list of all hourly values for the given key
        values = [hour.d[key] for hour in forecast.hourly().data]
        # calculate the mean, round to 2 significant figures and return
        return round(np.mean(values), 2)

    def _get_daytime_average(self, forecast, key):
        """
        Calculate the average for a given value during daytime hours (from sunrise to sunset)

        Parameters
        ----------
        forecast : Forecast Object
        key : String

        Returns
        -------
        float
        """
        # extract values from forecast
        try:
            values = [hour.d[key] for hour in forecast.hourly().data]
        except KeyError:
            return None

        # make a time series
        time = [hour.d['time'] for hour in forecast.hourly().data]
        ts = pd.Series(data=values, index=time)
        ts.index = pd.DatetimeIndex(ts.index.astype('datetime64[s]'))

        # get sunrise and sunset
        sunrise = dt.datetime.utcfromtimestamp(forecast.daily().data[0].d['sunriseTime'])
        sunset = dt.datetime.utcfromtimestamp(forecast.daily().data[0].d['sunsetTime'])

        # truncate timeseries
        ts = ts.truncate(before=sunrise, after=sunset)

        # return mean of truncated timeseries
        return round(ts.mean(), 2)

    @property
    def cache_folder(self):
        location_str = "{}_{}".format(round(self.location.latitude, 4),
                                      round(self.location.longitude, 4))

        forecast_folder = os.path.join(os.path.abspath(cfg.get('data', 'folder')),
                            'forecasts')
        location_folder = os.path.join(forecast_folder, location_str)

        for folder in [forecast_folder, location_folder]:
            if not os.path.exists(folder):
                print("This folder does not exist: {}, it will be created".format(folder))
                os.mkdir(folder)

        return location_folder

    def _pickle_path(self, date):
        filename = str(date) + '.pkl'
        path = os.path.join(self.cache_folder, filename)
        return path

    def _save_in_cache(self, f, date):
        """
        Save Forecast object to cache

        Parameters
        ----------
        f : Forecast
        date : datetime.date
        """

        pickle.dump(f, open(self._pickle_path(date), "wb"))

    def _load_from_cache(self, date):
        """
        Load Forecast object from cache

        Parameters
        ----------
        date : datetime.date

        Returns
        -------
        Forecast
        """
        path = self._pickle_path(date)
        if os.path.exists(path):
            return pickle.load(open(path, "rb"))
        else:
            return None



# -*- coding: utf-8 -*-

"""
A setuptools based setup module for opengrid.

Adapted from 
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
#with open(path.join(here, 'README.md'), encoding='utf-8') as f:
#    long_description = f.read()

import subprocess
if subprocess.call(["pip", "install","-r", path.join(here, "requirements.txt"), "-v", "--no-cache"]):
    raise Exception("Could not install dependencies")

setup(
    name='opengrid',
    version="0.4.4",
    description='Open-source algorithms for data-driven building analysis and control',
    #long_description=long_description,
    url='https://github.com/opengridcc/opengrid',
    author='Roel De Coninck and many others',
    author_email='roeldeconinck@gmail.com',
    license='Apache 2.0',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],

    keywords='algorithms buildings monitoring analysis control',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #py_modules=["tmpo.py"],

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    # Note: for creating the source distribution, they had to be included in the
    # MANIFEST.in as well. 
    package_data={
        'opengrid': ['LICENSE', 'README.md', 'requirements.txt', 'opengrid/library/houseprint/tests/test_saved_hp.hp',
                     'opengrid/library/tests/*', 'opengrid/library/tests/data/*', 'opengrid/library/tests/data/cache_day/*',
                     'notebooks/*'],
    },
)
